
(define (domain satellite)
 (:requirements :strips :equality :typing :fluents :durative-actions :timed-initial-literals)
 (:types satellite direction instrument mode)
 (:predicates 
        (on_board ?i - instrument ?s - satellite)
        (supports ?i - instrument ?m - mode)
        (pointing ?s - satellite ?d - direction)
        (power_avail ?s - satellite)
        (power_on ?i - instrument)
        (calibrated ?i - instrument)
        (have_image ?d - direction ?m - mode)
        (calibration_target ?i - instrument ?d - direction)
        (available ?s - satellite))
 (:functions 
        (data_capacity ?s - satellite)
        (data ?d - direction ?m - mode)
        (slew_time ?a ?b - direction)
        (data-stored)
        (fuel ?s - satellite)
        (fuel-used))

 (:durative-action turn_to
    :parameters (?s - satellite ?d_new - direction ?d_prev - direction)
    :duration (= ?duration (slew_time ?d_prev ?d_new))
    :condition (and (at start (pointing ?s ?d_prev)) (over all (not (= ?d_new ?d_prev))) 
               (at start (>= (fuel ?s) (slew_time ?d_new ?d_prev))))
    :effect (and (at end (pointing ?s ?d_new)) (at start (not (pointing ?s ?d_prev)))
            (at start (decrease (fuel ?s) (slew_time ?d_new ?d_prev)))
            (at start (increase (fuel-used) (slew_time ?d_new ?d_prev))))
 )

 (:durative-action switch_on
    :parameters (?i - instrument ?s - satellite)
    :duration (= ?duration 2)
    :condition (and (over all (on_board ?i ?s)) (at start (power_avail ?s)))
    :effect (and (at end (power_on ?i)) (at start (not (calibrated ?i)))
            (at start (not (power_avail ?s))))
 )
 
 (:durative-action switch_off
    :parameters (?i - instrument ?s - satellite)
    :duration (= ?duration 1)
    :condition (and (over all (on_board ?i ?s))(at start (power_on ?i)) )
    :effect (and (at start (not (power_on ?i))) (at end (power_avail ?s)))
 )

 (:durative-action calibrate
    :parameters (?s - satellite ?i - instrument ?d - direction)
    :duration (= ?duration 5)
    :condition (and (over all (on_board ?i ?s)) (over all (calibration_target ?i ?d))
               (at start (pointing ?s ?d)) (over all (power_on ?i)) (at end (power_on ?i)))
   :effect (at end (calibrated ?i))
 )

 (:durative-action take_image
    :parameters (?s - satellite ?d - direction ?i - instrument ?m - mode)
    :duration (= ?duration 10)
    :condition (and (over all (calibrated ?i))
                    (over all (on_board ?i ?s))
                    (over all (supports ?i ?m) )
                    (over all (power_on ?i))
                    (over all (pointing ?s ?d))
                    (at end (power_on ?i))
                    (over all (available ?s))
                    (at start (>= (data_capacity ?s) (data ?d ?m))))
    :effect (and (at start (decrease (data_capacity ?s) (data ?d ?m)))
                    (at end (have_image ?d ?m)) (at start (increase (data-stored) (data ?d ?m))))
 )

)

